# -*- coding: utf-8 -*-
"""Widgets."""

from ._widgets import (
    OQHistoryWidget,
    OQHistoryWidgetDefaultHeader,
    OQWidget,
    OQWidgetList,
    OQWidgetListDefaultHeader,
)

__all__ = [
    "OQWidget",
    "OQWidgetListDefaultHeader",
    "OQWidgetList",
    "OQHistoryWidgetDefaultHeader",
    "OQHistoryWidget",
]
